#!/usr/bin/env python

import sys, getopt, random, string, socket

# Ustalamy wstepne wartosci parametrow.

optlength = 0
optcount = 500
optsource = "../files/dictionary.txt"
opttarget = "inexistantdomains.txt"
optrandom = True
optdict = False
extensions = ['ac','ad','ae','aero','af','ag','ai','al','am','an','ao','aq','ar','as','at','au','aw','az','ba','bb','bd','be','bf','bg','bh','bi','biz','bj','bm','bn','bo','br','bs','bt','bv','bw','by','bz','ca','cc','cd','cd','cf','cg','ch','ci','ck','cl','cm','cn','co','com','coop','cr','cs','cu','cv','cx','cy','cz','de','dj','dk','dm','do','dz','ec','edu','ee','eg','eh','er','es','et','eu','fi','fj','fk','fm','fo','fr','ga','gb','gd','ge','gf','gg','gh','gi','gl','gm','gn','gov','gp','gq','gr','gs','gt','gu','gw','gy','hk','hm','hn','hr','ht','hu','id','ie','il','im','in','info','int','io','iq','ir','is','it','je','jm','jo','jp','ke','kg','kh','ki','km','kn','kp','kr','kw','ky','kz','la','lb','lc','li','lk','lr','ls','lt','lu','lv','ly','ma','mc','md','mg','mh','mil','mk','ml','mm','mn','mo','mp','mq','mr','ms','mt','mu','museum','mv','mw','mx','my','mz','na','name','nc','ne','net','nf','ng','ni','nl','no','np','nr','nu','nz','om','org','pa','pe','pf','pg','ph','pk','pl','pm','pn','pr','ps','pt','pw','py','qa','re','ro','ru','rw','sa','sb','sc','sd','se','sg','sh','si','sj','sk','sl','sm','sn','so','sr','st','su','sv','sy','sz','tc','td','tf','tg','th','tj','tk','tm','tn','to','tp','tr','travel','tt','tv','tw','tz','ua','ug','uk','um','us','uy','uz','va','vc','ve','vg','vi','vn','vu','wf','ws','ye','yt','yu','za','zm','zw']
errcount = 0

# Funkcja do wyswietlania sposobu uzycia programu.

def usage(level):
    print "FeedHarvey.py: Sposob uzycia:"
    print "feedharvey.py [-h|--help] [[-r|--random] | [-d|--dictionary]]"
    print "              [-l|--length <dlugosc domen>] [-c|--count <liczba domen>]"
    print "              [-s|--source <plik zrodlowy>] [-t|--target <plik docelowy>]"
    if level > 0:
        print
        print "              -h, --help       Wyswietla pomoc (to co widzisz)."
        print "              -r, --random     Generowanie domen ze znakow losowych."
        print "              -d, --dictionary Generowanie domen z pliku slownika."
        print "                               Domyslnie: -r."
        print "              -l, --length     Dlugosc domen."
        print "                               Dla -r: maksymalna dlugosc bez rozszerzenia."
        print "                               Domyslnie: 10."
        print "                               Dla -d: liczba slow ze slownika."
        print "                               Domyslnie: 1."
        print "              -c, --count      Liczba wygenerowanych domen."
        print "                               Domyslnie: 500."
        print "              -s, --source     Sciezka do pliku zrodlowego."
        print "                               Domyslnie: ../files/dictionary.txt."
        print "              -t, --target     Sciezka do pliku docelowego."
        print "                               Domyslnie: inexistantdomains.txt."
        print
        print "Przyklad uzycia:"
        print
        print "  feedharvey.py -d -l 2 -c 100 -s dict.txt -d domains.txt"
        print "  - generuje domeny w oparciu o slownik dict.txt, skladajac je z dwoch slow,"
        print "    zapisuje 100 sprawdzonych domen w pliku domains.txt."

# Odczytujemy parametry podane z linii polecen.

try:
    opts, args = getopt.getopt(sys.argv[1:], "hrdl:c:s:t:", ["help", "random", "dictionary", "length=", "count=", "source=", "target="])
except getopt.GetOptError:
    usage(0)
    sys.exit(2)
for o, a in opts:
    if o in ("-h", "--help"):
        usage(1)
        sys.exit(0)
    if o in ("-r", "--random"):
        optrandom = True
        optdict = False
    if o in ("-d", "--dictionary"):
        optdict = True
        optrandom = False
    if o in ("-l", "--length"):
        optlength = int(a)
    if o in ("-c", "--count"):
        optcount = int(a)
    if o in ("-s", "--source"):
        optsource = a
    if o in ("-t", "--target"):
        opttarget = a

# Sprawdzamy poprawnosc pobranych parametrow.

# Nie mozna generowac jednoczesnie ciagow losowych i w oparciu o slownik.

if optrandom == True and optdict == True:
    print "FeedHarvey.py: Nie mozesz uzyc jednoczesnie opcji -r i -d."
    sys.exit(1)

# Jesli dlugosc jest zerowa (nie zostala podana lub zostala podana blednie
# to przyjeta zostanie dlugosc domyslna, zalezna od opcji (-r czy -d).

if optlength == 0:
    if optrandom == True:
        optlength = 10
    elif optdict == True:
        optlength = 1

# Jesli liczba generowanych domen jest zerowa, oznacza to ze zostal podany
# nieprawidlowy parametr.

if optcount == 0:
    print "FeedHarvey.py: Po opcji -c musisz podac liczbe naturalna wieksza od 0."
    sys.exit(1)

# Plik zrodlowy (slownika) jest potrzebny tylko przy opcji -r.
# Sprawdzamy, czy plik mozna otworzyc (czy istnieje).

if optdict == True:
    try:
        fpsource = open(optsource, "r")
    except IOError:
        print "FeedHarvey.py: Podaj prawidlowa sciezke do pliku po opcji -s."
        sys.exit(1)
    data = fpsource.read()
    data = string.split(data, "\n")
    data = filter (None, map(string.strip, data))

# Sprawdzamy, czy mozemy zapisywac do pliku docelowego.

try:
    fptarget = open(opttarget, "w")
except IOError:
    print "FeedHarvey.py: Nie mozna otworzyc wskazanego pliku docelowego do zapisu."
    sys.exit(1)

# Sprawdzilismy parametry, teraz mozemy przejsc do wlasciwego generowania.

for i in range(1, optcount):
    if optrandom == True:
        body = string.join(map(lambda i: chr(random.randint(97, 122)), range(random.randint(optlength/2, optlength))), "")
    elif optdict == True:
        body = ""
        for j in range(0, optlength):
            body = body + random.choice(data)
    ext = extensions[random.randint(0,len(extensions)-1)]
    domain = body + "." + ext
    print "FeedHarvey.py: Generated domain:", domain
    if len(domain) > 64:
        print "               Domain too long. Skipping."
        errcount = errcount + 1
    else:
        try:
            ip = socket.gethostbyname(domain)
        except socket.gaierror:
            fptarget.write(domain)
            fptarget.write("\n")
            print "               Domain written to file."
        else:
            print "               Domain exists (" + ip + "). Skipping."
            errcount = errcount + 1
    if errcount > optcount / 5:
        print "FeedHarvey.py: More than 20% erroneous domains found."
        print "               Something's wrong with parameters. Exiting."
        fptarget.close()
        sys.exit(1)
fptarget.close()
